subroutine lsvecopt(evalf,evalg,stp,stmin,stmax,m,f0,g0,ftol,gtol,nfev,ngev,info)

	implicit none
	
	! SCALAR ARGUMENTS
	integer, intent(in) :: m
	integer, intent(out) :: nfev, ngev, info
	real(kind=8), intent(in) :: stmin, stmax, ftol, gtol
	real(kind=8), intent(inout) :: stp
	
	! ARRAY ARGUMENTS
	real(kind=8), intent(in) ::f0(m), g0(m)
	
	! LOCAL SCALARS
	real(kind=8), parameter :: zero = 0.0d0, one = 1.0d0, two = 2.0d0
	integer :: ind, indprev, i, outiter, initer, totiniter, maxoutiter,  & 
	           flag
	real(kind=8) :: f, g, xtol, ftolinner, gtolinner,  &
									maxg0, maxg, gtest, ftest, stpmin, stpmax
	logical :: sdc, cc, brackt, iprint

	! LOCAL ARRAYS
	integer :: isave(3)
	real(kind=8) :: dsave(13)
	character(len=60) :: task
	
	! INTERFACE OF MORE AND THUENTE SUBROUTINES 
	interface
	
		! SUBROUTINE dcsrch:
		subroutine dcsrch(stp,f,g,ftol,gtol,xtol,task,stpmin,stpmax,isave,dsave)
		! SCALAR ARGUMENTS
		real(kind=8), intent(in) :: ftol, gtol, xtol, stpmin, stpmax
		real(kind=8), intent(inout) :: f, g, stp
		character(len=60), intent(inout) :: task    
		! ARRAY ARGUMENTS    
		integer, intent(inout) :: isave(2)
		real(kind=8), intent(inout) :: dsave(13)
		end subroutine dcsrch
							
		! SUBROUTINE dcstep:
		subroutine dcstep(stx,fx,dx,sty,fy,dy,stp,fp,dp,brackt,stpmin,stpmax)
		! SCALAR ARGUMENTS
		real(kind=8), intent(in) :: fp,dp, stpmin,stpmax
		real(kind=8), intent(inout) :: stx,fx,dx,sty,fy,dy,stp
		logical, intent(inout) :: brackt
		end subroutine dcstep
	
	end interface
	
	!	info:
	!	0: stp satisfies the strong Wolfe conditions.
	!	1: stp = stpmin   
	!	2: stp = stpmax   
	!	3: rouding erros prevent progress
	!	4: the working interval is too small
	!	5: the number of iterations is exhausted
		
	!---------------------------------------------------------------------   
	!     Initialization
	!---------------------------------------------------------------------    

	xtol = 1.d-20

	ftolinner = min( 1.1d0 * ftol, 0.75d0 * ftol + 0.25d0 * gtol )
	gtolinner = max( 0.9d0 * gtol, 0.25d0 * ftol + 0.75d0 * gtol )
	
	maxoutiter = 100
	
	stpmin = stmin
	stpmax = stmax

	brackt = .false.
	
	iprint = .false.
	
	! Print problem information
	
	if ( iprint ) then
			write(*,1000)
			write(*,1010) ftol, gtol
			write(*,1020) m, stpmin, stpmax
	end if
	
	! Check for errors in input data
	
	if ( ftol >= gtol ) then
			write(*,1021)
			info = - 1
			return
	end if
	
	if ( stp < stpmin ) then
			write(*,1022)
			info = - 1
			return
	end if
	
    if ( stp > stpmax ) then 
			write(*,1023)
			info = - 1
			return
	end if

	! Counters

	outiter = 0
	totiniter = 0
	nfev = 0
	ngev = 0

  ! Compute maxg0
  
	maxg0 = maxval(g0)
	
	! Check if maxg0 is negative, and define ftest and gtest

	if ( maxg0 >= zero ) then
			write(*,1030)
			info = - 1
			return
	end if
	
	gtest = -gtol * maxg0
	ftest =  ftol * maxg0	
		
	ind = minloc(g0,dim=1)

	if ( iprint ) then
			write(*,1040) gtest
			write(*,1050)
	end if

	
	!---------------------------------------------------------------------  
	!     Main loop
	!---------------------------------------------------------------------     

	Main_loop: do
	
	!---------------------------------------------------------------------  
	!     Prepare the iteration
	!--------------------------------------------------------------------- 

		if ( outiter == 0  .or. task(1:4) == 'CONV' ) then

				sdc = .true.     
				do i = 1, m
						call evalf(stp,f,i)
						nfev = nfev + 1		
						
						if ( f > f0(i) + ftest * stp ) then
								ind = i
								stpmax = stp    
								sdc = .false.
								brackt = .true.
								exit
						end if		
				end do
				
				cc = .false.
				if ( sdc ) then
						do i = 1, m
							call evalg(stp,g,i)
							ngev = ngev + 1
							
							if ( i == 1 ) then 
									maxg = g
							else								
									maxg = max( maxg , g )
							end if 
							
							
							if ( g > gtest ) then
									ind = i
									stpmax = stp
									brackt = .true.
									exit
							end if	
										
						end do	
						
						if ( abs(maxg) <= gtest ) then
								cc = .true.
						end if
				end if 
		
  	end if


		
		!-------------------------------------------------------------------
		!     Print information
		!-------------------------------------------------------------------
		
		if ( iprint ) then
				if ( outiter == 0 )	then
						write(*,1060) outiter, ind, stp, stpmin, stpmax, sdc, cc, brackt
				else
						write(*,1070) outiter, ind, stp, stpmin, stpmax, sdc, cc, brackt, initer
				end if
		end if

		!-------------------------------------------------------------------
		!     Test stopping criteria
		!-------------------------------------------------------------------
		
		! Test the sufficient descent condition and the curvature condition
		
		if ( sdc .and. cc ) then
				if ( iprint ) then
						write(*,1080) stp
						write(*,1090) nfev, ngev
				end if
				info = 0
				return
		end if

		! Test for stopping at stp = stpmin   

		if ( outiter > 0 .and. task == 'WARNING: STP = STPMIN' ) then
				if ( iprint ) then
						write(*,1100) stp
						write(*,1090) nfev, ngev
				end if
				info = 1
				return
		end if
		
		! Test for stopping at stp = stpmax  

		if ( outiter > 0 .and. ind == indprev .and.	                       &
				 task == 'WARNING: STP = STPMAX' ) then
				if ( iprint ) then
						write(*,1110) stp
						write(*,1090) nfev, ngev
				end if
				info = 2
				return
		end if
		
		! Test whether rouding erros prevent progress

		if ( outiter > 0 .and. ind == indprev .and.												 &
		     task == 'WARNING: ROUNDING ERRORS PREVENT PROGRESS' ) then
		    if ( iprint ) then
						write(*,1120) stp
						write(*,1090) nfev, ngev
				end if
				info = 3
				return
		end if
		
		! Test whether the working interval is too small

		if ( outiter > 0 .and. ind == indprev .and.                        &
		     task == 'WARNING: XTOL TEST SATISFIED' ) then
		    if ( iprint ) then
						write(*,1130) stp
						write(*,1090) nfev, ngev
				end if
				info = 4
				return
		end if
		
		! Test whether the number of iterations is exhausted
		
		if ( outiter > maxoutiter ) then
				if ( iprint ) then
						write(*,1140) stp
						write(*,1090) nfev, ngev
				end if
				info = 5
				return
		end if
		
		!-------------------------------------------------------------------  
		!     Iteration
		!------------------------------------------------------------------- 

		outiter = outiter + 1

		f = f0(ind)
		g = maxg0
		task = 'START'
		initer = 0
	
		!------------------------------------------------------------------- 
		!     Call the Moré and Thuente line search
		!------------------------------------------------------------------- 
		 
		! J. J. Moré and D. J. Thuente, Line Search Algorithms with Guaranteed
		! Sufficient Decrease, ACM Trans. Math. Softw., 20 (1994), pp. 286–307.
		
		Inner_loop: do 
		
			if ( task /= 'START' ) initer = initer + 1
						
			call dcsrch(stp,f,g,ftolinner,gtolinner,xtol,task,stpmin,stpmax,isave,dsave)
 
			if ( initer > 0 .and. isave(1) == 0 .and. task(1:4) /= 'WARN' ) then
			
					do i = 1,m
							if ( i == ind ) cycle
							
							call evalf(stp,f,i)
							nfev = nfev + 1
							
							if ( f > f0(i) + ftest * stp ) then
									sdc = .false.
									brackt = .true.
									ind = i
									stpmax = stp
									exit Inner_loop
							end if
							
							call evalg(stp,g,i)
							ngev = ngev + 1
		
							if ( g > gtest ) then
									cc = .false.
									brackt = .true.
									ind = i
									stpmax = stp
									exit Inner_loop
							end if
					end do
			end if

			! Test for convergence of the inner method
			
			if ( task(1:4) == 'WARN' .or. task(1:4) == 'CONV' ) exit Inner_loop	
			
			! Evaluate f and g
			
			call evalf(stp,f,ind)
			call evalg(stp,g,ind)
			nfev = nfev + 1
			ngev = ngev + 1		
			
		end do Inner_loop
		
		!------------------------------------------------------------------- 
		
		totiniter = totiniter + initer
		
		! In case of warnings, save the current value of ind for use it in 
		! the stopping criteria tests
		
		if ( task == 'WARNING: ROUNDING ERRORS PREVENT PROGRESS' .or. &
			   task == 'WARNING: XTOL TEST SATISFIED'              .or. &
			   task == 'WARNING: STP = STPMAX'  ) then
			   
			   indprev = ind
			   
					do i = 1,m
							if ( i == ind ) cycle
							
							call evalf(stp,f,i)
							nfev = nfev + 1
							
							if ( f > f0(i) + ftest * stp ) then
									sdc = .false.
									brackt = .true.
									ind = i
									stpmax = stp
									exit
							end if
							
							call evalg(stp,g,i)
							ngev = ngev + 1
		
							if ( g > gtest ) then
									cc = .false.
									brackt = .true.
									ind = i
									stpmax = stp
									exit
							end if
					end do
					
		end if		     
		
	!--------------------------------------------------------------------- 
	!     Iterate
	!---------------------------------------------------------------------    

	end do Main_loop

	!--------------------------------------------------------------------- 
	!     End of main loop
	!---------------------------------------------------------------------
	
	! Non-executable statements
	
	1000 format(/,1X,'-----------------------------------------------------',/,1X,& 
					 'lsvecopt: A Wolfe line search for vector Optimization',/,1X,& 
					 '-----------------------------------------------------')

    1010 format(/,1X,'Sufficient descent condition tolerance:',1X,1P,D7.1,&
              /,1X,'Curvature condition tolerance:         '1X,1P,D7.1)
 
	1020 format(/,1X,'Number of functions:',1X,I3,/,1X,'stpmin =',1X,1P,D7.1,/,1X,'stpmax =',1X,1P,D7.1)	
	
	1021 format(/,'Error of lsvecopt: ftol >= gtol.')
	
	1022 format(/,'Error of lsvecopt: stp < stpmin.')
	
	1023 format(/,'Error of lsvecopt: stp > stpmax.')
	
	1030 format(/,'Error of lsvecopt: the search direction is not a descent direction.')
	
	1040 format(/,1X,'Curvature condition test: -gtol * maxg0 =',1X,1P,D7.1)
	
	1050 format(/,2X,'out',1X,'ind',4X,'stp',5X,'stpmin',4X,'stpmax',3X,'SDC',3X,'CC',2X,'brackt',1X,'inner')
	
	1060 format(1X,I3,1X,I3,2X,1P,D8.2,2X,1P,D8.2,2X,1P,D8.2,3X,L1,4X,L1,5X,L1,6X,'-')
	
	1070 format(1X,I3,1X,I3,2X,1P,D8.2,2X,1P,D8.2,2X,1P,D8.2,3X,L1,4X,L1,5X,L1,4X,I3)		
	
	1080 format(/,1X,'Flag of lsvecopt: a point satisfying the sufficient descent condition',/,19X,&
	                 'and the curvature condition was found',/,/,1X,'Final step: ',1P,D8.2)						
	
	1090 format(/,1X,'Number of functions evaluations:   ',I4,/,1X,'Number of derivatives evaluations: ',I4)
	
	1100 format(/,1X,'Flag of lsvecopt: the final step is equal to stpmin',/,/,1X,'Final step: ',1P,D8.2)
	
	1110 format(/,1X,'Flag of lsvecopt: the final step is equal to stpmax',/,/,1X,'Final step: ',1P,D8.2)
	
	1120 format(/,1X,'Flag of lsvecopt: rouding erros prevent progress',/,/,1X,'Final step: ',1P,D8.2)
	
	1130 format(/,1X,'Flag of lsvecopt: the working interval is too small',/,/,1X,'Final step: ',1P,D8.2)
	
	1140 format(/,1X,'Flag of lsvecopt: maximum of iterations reached',/,/,1X,'Final step: ',1P,D8.2)

end subroutine lsvecopt
